@@ -3,7 +3,6 @@
 {-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE DeriveGeneric #-}
 {-# LANGUAGE MagicHash #-}
-{-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE StrictData #-}
 {-# LANGUAGE UnboxedTuples #-}
 {-# OPTIONS_GHC -funbox-strict-fields #-}
@@ -70,7 +69,7 @@ data Int128 = Int128
   { int128Hi64 :: !Word64
   , int128Lo64 :: !Word64
   }
-  deriving (Eq, Data, Generic, Ix, Typeable)
+  deriving (Eq, Data, Generic, Ix)
 
 instance Hashable Int128 where
   hashWithSalt s (Int128 a1 a2) = s `hashWithSalt` a1 `hashWithSalt` a2
@@ -234,7 +233,14 @@ toWord32 (Int128 _ w) = fromIntegral w
 -- Functions for `Ord` instance.
 
 compare128 :: Int128 -> Int128 -> Ordering
-compare128 a b = compare (toInteger128 a) (toInteger128 b)
+compare128 (Int128 a1 a0) (Int128 b1 b0)
+  | aIsNeg == bIsNeg = compare a1 b1 <> compare a0 b0
+  | bIsNeg = GT
+  | otherwise = LT
+  where
+    aIsNeg = isNeg a1
+    bIsNeg = isNeg b1
+    isNeg = (>= 0x8000000000000000)
 
 -- -----------------------------------------------------------------------------
 -- Functions for `Enum` instance.
